<?php
// +--------------------------------------------------------------------------+
// | Image_Graph                                                              |
// +--------------------------------------------------------------------------+
// | Copyright (C) 2003, 2004 Jesper Veggerby                                 |
// | Email         pear.nosey@veggerby.dk                                     |
// | Web           http://pear.veggerby.dk                                    |
// | PEAR          http://pear.php.net/package/Image_Graph                    |
// +--------------------------------------------------------------------------+
// | This library is free software; you can redistribute it and/or            |
// | modify it under the terms of the GNU Lesser General Public               |
// | License as published by the Free Software Foundation; either             |
// | version 2.1 of the License, or (at your option) any later version.       |
// |                                                                          |
// | This library is distributed in the hope that it will be useful,          |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of           |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU        |
// | Lesser General Public License for more details.                          |
// |                                                                          |
// | You should have received a copy of the GNU Lesser General Public         |
// | License along with this library; if not, write to the Free Software      |
// | Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA |
// +--------------------------------------------------------------------------+

/**
 * Image_Graph - PEAR PHP OO Graph Rendering Utility.
 * @package Image_Graph
 * @subpackage Axis     
 * @category images
 * @copyright Copyright (C) 2003, 2004 Jesper Veggerby Hansen
 * @license http://www.gnu.org/licenses/lgpl.txt GNU Lesser General Public License
 * @author Jesper Veggerby <pear.nosey@veggerby.dk>
 * @version $Id: Category.php,v 1.4 2004/10/21 20:30:39 nosey Exp $
 */ 

/**
 * Include file Image/Graph/Axis.php
 */
require_once 'Image/Graph/Axis.php';

/**
 * A normal axis thats displays labels with a 'interval' of 1.
 * This is basically a normal axis where the range is
 * the number of labels defined, that is the range is explicitly defined
 * when constructing the axis. 
 */
class Image_Graph_Axis_Category extends Image_Graph_Axis 
{
    
    /** The labels shown on the axis
     * @var array
     * @access private
     */
    var $_labels = false;

    /**
     * Image_Graph_Axis_Category [Constructor].
     * @param int $type The type (direction) of the Axis
     */
    function &Image_Graph_Axis_Category($type = IMAGE_GRAPH_AXIS_X)
    {
        parent::Image_Graph_Axis($type);
        $this->_labels = array();
    }

    /**
     * Gets the minimum value the axis will show.
     * This is always 0
     * @return double The minumum value
     * @access private
     */
    function _getMinimum()
    {
        return 0;
    }

    /**
     * Gets the maximum value the axis will show.
     * This is always the number of labels passed to the constructor.
     * @return double The maximum value
     * @access private
     */
    function _getMaximum()
    {
        return count($this->_labels) - ($this->_pushValues ? 0 : 1);
    }

    /**
     * Sets the minimum value the axis will show.
     * A minimum cannot be set on a SequentialAxis, it is always 0.
     * @param double Minimum The minumum value to use on the axis
     * @access private
     */
    function _setMinimum($minimum)
    {
    }

    /**
     * Sets the maximum value the axis will show
     * A maximum cannot be set on a SequentialAxis, it is always the number
     * of labels passed to the constructor.
     * @param double Maximum The maximum value to use on the axis
     * @access private
     */
    function _setMaximum($maximum)
    {
    }
    
    /**
     * Forces the minimum value of the axis
     * A minimum cannot be set on a SequentialAxis, it is always 0.
     * @param double $minimum The minumum value to use on the axis
     */
    function forceMinimum($minimum)
    {
    }

    /**
     * Forces the maximum value of the axis
     * A maximum cannot be set on a SequentialAxis, it is always the number
     * of labels passed to the constructor.
     * @param double $maximum The maximum value to use on the axis
     */
    function forceMaximum($maximum)
    {
    }

    /**
     * Sets an interval for when labels are shown on the axis.
     * The label interval cannot be set for a SequentialAxis
     * @param double $labelInterval The interval with which labels are shown
     */
    function setLabelInterval($labelInterval = 'auto')
    {
    }

    /**
     * Preprocessor for values, ie for using logarithmic axis
     * @param double $value The value to preprocess
     * @return double The preprocessed value
     * @access private
     */
    function _value($value)
    {
        $the_value = array_search($value, $this->_labels);
        if ($the_value !== false) {
            return $the_value + ($this->_pushValues ? 0.5 : 0);
        } else {
            return 0;
        }
    }

    /**
     * Get the interval with which labels are shown on the axis.
     * This is always 1.
     * @return double The label interval
     * @access private
     */
    function _labelInterval()
    {
        return 1;
    }

    /**
     * Get the minor label interval with which axis label ticks are drawn.
     * For a sequential axis this is always disabled (i.e false)
     * @return double The minor label interval, always false
     * @access private
     */
    function _minorLabelInterval()
    {
        return false;
    }
    
    /** 
     * Get the size in pixels of the axis.
     * For an x-axis this is the width of the axis including labels, and for an
     * y-axis it is the corrresponding height
     * @return int The size of the axis
     * @access private 
     */
     function _size()
     {
        if (!$this->_font) {
            $this->_font = $GLOBALS['_Image_Graph_font'];
        }

        $maxSize = 0;
        reset($this->_labels);        
        while (list($id, $label) = each($this->_labels)) {        
            $labelPosition = $this->_point($label);

            if (is_object($this->_dataPreProcessor)) {
                $labelText = $this->_dataPreProcessor->_process($label);
            } else {
                $labelText = $label;
            }

            if ($this->_type == IMAGE_GRAPH_AXIS_Y) {
                $maxSize = max($maxSize, $this->_font->width($labelText));
            } else {
                $maxSize = max($maxSize, $this->_font->height($labelText));
            }
        }
        return $maxSize +3;
    }
    
    /**
     * Apply the dataset to the axis.
     * This calculates the order of the categories, which is very important for fx.
     * line plots, so that the line does not "go backwards", consider these X-sets:
     * 1: (1, 2, 3, 4, 5, 6)
     * 2: (0, 1, 2, 3, 4, 5, 6, 7)
     * If they are not ordered, but simply appended, the categories on the axis would
     * be:
     * X: (1, 2, 3, 4, 5, 6, 0, 7)
     * Which would render the a line for the second plot to show incorrectly. Instead
     * this algorithm, uses and 'value-is-before' method to see that the 0 is before a
     * 1 in the second set, and that it should also be before a 1 in the X set. Hence:
     * X: (0, 1, 2, 3, 4, 5, 6, 7)
     * @param Image_Graph_Dataset $dataset The dataset
     * @access private
     */
    function _applyDataset(&$dataset)
    {
        $dataset->_reset();
        while ($point = $dataset->_next()) {
            if ($this->_type == IMAGE_GRAPH_AXIS_X) {
                $data = $point['X'];
            } else {
                $data = $point['Y'];
            }
            if (!in_array($data, $this->_labels)) {
                $newLabels[] = $data;
                //$this->_labels[] = $data;
            }     
            $allLabels[] = $data;           
        }
        if (count($this->_labels) == 0) {
            $this->_labels = $newLabels;
        } elseif (is_array($newLabels)) {
            // get all intersecting labels
            $intersect = array_intersect($allLabels, $this->_labels);
            // traverse all new and find their relative position withing the intersec,
            // fx value X0 is before X1 in the intersection, which means that X0 should
            // be placed before X1 in the label array
            while (list($id, $newLabel) = each($newLabels)) {
                $key = array_search($newLabel, $allLabels);
                reset($intersect);
                $this_value = false;
                // intersect indexes are the same as in allLabels!
                $first = true;
                while ((list($id, $value) = each($intersect)) and ($this_value === false)) {
                    if (($first) and ($id > $key)) {
                        $this_value = $value;
                    } elseif ($id >= $key) {
                        $this_value = $value;
                    }
                    $first = false;
                }                 
                if ($this_value === false) {
                    // the new label was not found before anything in the intersection -> append it
                    $this->_labels[] = $newLabel;
                } else {
                    // the new label was found before $this_value in the intersection, 
                    // insert the label before this position in the label array
                    $key = array_search($this_value, $this->_labels);
                    $pre = array_slice($this->_labels, 0, $key);
                    $pre[] = $newLabel;
                    $post = array_slice($this->_labels, $key);
                    $this->_labels = array_merge($pre, $post);
                }
            }
        }
    }

    /**
     * Get next label point
     * @param doubt $point The current point, if omitted or false, the first is returned
     * @return double The next label point
     * @access private
     */
    function _getNextLabel($currentLabel = false)
    {               
        if ($currentLabel === false) {
            reset($this->_labels);
        }
        if (list($id, $label) = each($this->_labels)) {
            return $label;
        } else  {
            return false;
        }            
    }
         
    /**
     * Is the axis numeric or not?
     * return bool True if numeric, false if not
     * @access private
     */
    function _isNumeric() {
        return false;
    }
    
    /**
     * Output the axis
     * @access private
     */
    function _done()
    {
        Image_Graph_Element::_done();

        if (!$this->_font) {
            $this->_font = $GLOBALS['_Image_Graph_font'];
        }

        reset($this->_labels);
        
        while (list($id, $label) = each($this->_labels)) {        
            $labelPosition = $this->_point($label);
                
            if (is_object($this->_dataPreProcessor)) {
                $labelText = $this->_dataPreProcessor->_process($label);
            } else {
                $labelText = $label;
            }

            if ($this->_type == IMAGE_GRAPH_AXIS_Y) {
                $this->write($this->_right - 3, $labelPosition, $labelText, IMAGE_GRAPH_ALIGN_CENTER_Y | IMAGE_GRAPH_ALIGN_RIGHT);
            } else {
                $this->write($labelPosition, $this->_top + 3, $labelText, IMAGE_GRAPH_ALIGN_CENTER_X | IMAGE_GRAPH_ALIGN_TOP);
            }

            if ($this->_type == IMAGE_GRAPH_AXIS_Y) {
                ImageLine($this->_canvas(), $this->_right, $labelPosition, $this->_right + 6, $labelPosition, $this->_getLineStyle());
            } else {
                ImageLine($this->_canvas(), $labelPosition, $this->_top, $labelPosition, $this->_top - 6, $this->_getLineStyle());
            }
        }

        if ($this->_type == IMAGE_GRAPH_AXIS_Y) {
            ImageLine($this->_canvas(), $this->_right, $this->_top, $this->_right, $this->_bottom, $this->_getLineStyle());
            if ($this->_showArrow) {
                $arrow[] = $this->_right - 5;
                $arrow[] = $this->_top + 8;
                $arrow[] = $this->_right;
                $arrow[] = $this->_top;
                $arrow[] = $this->_right + 5;
                $arrow[] = $this->_top + 8;
                ImageFilledPolygon($this->_canvas(), $arrow, count($arrow) / 2, $this->_getFillStyle());
                ImagePolygon($this->_canvas(), $arrow, count($arrow) / 2, $this->_getLineStyle());
            }
        } else {
            ImageLine($this->_canvas(), $this->_left, $this->_top, $this->_right, $this->_top, $this->_getLineStyle());
            if ($this->_showArrow) {
                $arrow[] = $this->_right - 8;
                $arrow[] = $this->_top + 5;
                $arrow[] = $this->_right;
                $arrow[] = $this->_top;
                $arrow[] = $this->_right - 8;
                $arrow[] = $this->_top - 5;
                ImageFilledPolygon($this->_canvas(), $arrow, count($arrow) / 2, $this->_getFillStyle());
                ImagePolygon($this->_canvas(), $arrow, count($arrow) / 2, $this->_getLineStyle());
            }
        }
    }

}

?>