<?php
// +--------------------------------------------------------------------------+
// | Image_Graph                                                              |
// +--------------------------------------------------------------------------+
// | Copyright (C) 2003, 2004 Jesper Veggerby                                 |
// | Email         pear.nosey@veggerby.dk                                     |
// | Web           http://pear.veggerby.dk                                    |
// | PEAR          http://pear.php.net/package/Image_Graph                    |
// +--------------------------------------------------------------------------+
// | This library is free software; you can redistribute it and/or            |
// | modify it under the terms of the GNU Lesser General Public               |
// | License as published by the Free Software Foundation; either             |
// | version 2.1 of the License, or (at your option) any later version.       |
// |                                                                          |
// | This library is distributed in the hope that it will be useful,          |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of           |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU        |
// | Lesser General Public License for more details.                          |
// |                                                                          |
// | You should have received a copy of the GNU Lesser General Public         |
// | License along with this library; if not, write to the Free Software      |
// | Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA |
// +--------------------------------------------------------------------------+

/**
 * Image_Graph - PEAR PHP OO Graph Rendering Utility.
 * @package Image_Graph
 * @subpackage Axis     
 * @category images
 * @copyright Copyright (C) 2003, 2004 Jesper Veggerby Hansen
 * @license http://www.gnu.org/licenses/lgpl.txt GNU Lesser General Public License
 * @author Jesper Veggerby <pear.nosey@veggerby.dk>
 * @version $Id: Radar.php,v 1.6 2004/10/18 20:53:43 nosey Exp $
 */ 

/**
 * Include file Image/Graph/Axis/Multidimensional.php
 */
require_once 'Image/Graph/Axis/Multidimensional.php';

/**
 * Displays an 'X'-axis in a radar plot chart.
 * This axis maps the number of elements in the dataset to a angle (from 0-360 degrees).
 * Displaying the axis consist of drawing a number of lines from center to the edge of 
 * the 'circle' than encloses the radar plot. The labels are drawn on the 'ends' of these
 * radial lines.   
 */
class Image_Graph_Axis_Radar extends Image_Graph_Axis_Multidimensional 
{

    /**
     * Gets the minimum value the axis will show
     * @return double The minumum value
     * @access private
     */
    function _getMinimum()
    {
        return $this->_minimum;
    }

    /**
     * Gets the maximum value the axis will show
     * @return double The maximum value
     * @access private
     */
    function _getMaximum()
    {
        return $this->_maximum;
    }

    /**
     * Get the step each pixel on the canvas will represent on the axis.     
     * @return double The step a pixel represents
     * @access private
     */
    function _delta()
    {
        if (abs($this->_getMaximum() - $this->_getMinimum()) == 0) {
            return 0;
        }

        return 360 / ($this->_getMaximum() - $this->_getMinimum());
    }

    /**
     * Get the pixel position represented by a value on the canvas
     * @param double $value the value to get the pixel-point for 
     * @return double The pixel position along the axis
     * @access private
     */
    function _point($value)
    {
        return (90 + (int) ($this->_value($value) * $this->_delta())) % 360;
    }

    /**
     * Get the interval with which labels are shown on the axis.
     * For a radar plot this is always 1
     * @return double The label interval always 1
     * @access private
     */
    function _labelInterval()
    {
        return 1;
    }

    /**
     * Get the minor label interval with which axis label ticks are drawn.
     * For a radar plot this is always disabled (i.e false)
     * @return double The minor label interval, always false
     * @access private
     */
    function _minorLabelInterval()
    {
        return false;
    }

    /** 
     * Get the size in pixels of the axis.
     * For a radar plot this is always 0
     * @return int The size of the axis
     * @access private 
     */
    function _size()
    {
        return 0;
    }

    /**
     * Output the axis
     * @access private
     */
    function _done()
    {
        Image_Graph_Element::_done();

        if (!$this->_font) {
            $this->_font = $GLOBALS['_Image_Graph_font'];
        }

        $labelInterval = $this->_labelInterval();
        $value = $this->_getMinimum();

        $centerX = (int) (($this->_left + $this->_right) / 2);
        $centerY = (int) (($this->_top + $this->_bottom) / 2);

        $radius = min($this->height(), $this->width()) / 2;

        while ($value < $this->_getMaximum()) {
            $endPoint = array ('X' => $value, 'Y' => false);
            $dX = $this->_pointX($endPoint);
            $dY = $this->_pointY($endPoint);

            if (!$this->_font) {
                $this->_font = $GLOBALS['_Image_Graph_font'];
            }

            if (is_object($this->_dataPreProcessor)) {
                $labelText = $this->_dataPreProcessor->_process($value);
            } else {
                $labelText = $value;
            }

            if ((abs($dX - $centerX) < 1.5) and ($dY < $centerY)) {
                $align = IMAGE_GRAPH_ALIGN_BOTTOM + IMAGE_GRAPH_ALIGN_CENTER_X;
            }
            elseif ((abs($dX - $centerX) < 1.5) and ($dY > $centerY)) {
                $align = IMAGE_GRAPH_ALIGN_TOP + IMAGE_GRAPH_ALIGN_CENTER_X;
            }
            elseif ($dX < $centerX) {
                $align = IMAGE_GRAPH_ALIGN_RIGHT + IMAGE_GRAPH_ALIGN_CENTER_Y;
            } else {
                $align = IMAGE_GRAPH_ALIGN_LEFT + IMAGE_GRAPH_ALIGN_CENTER_Y;
            }
            $this->write($dX, $dY, $labelText, $align);

            ImageLine($this->_canvas(), $centerX, $centerY, $dX, $dY, $this->_getLineStyle());
            $value += $labelInterval;
        }
    }

}

?>